/** The GoCanvas class is what displays a GoBoard data structure. */


import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.applet.*;
import java.util.*;


class GoCanvas extends Canvas
{
  GoBoard       board;
  GoGame        game;
  Font          font;
  MouseAdapter  mousey;
  Cursor        defaultcursor, handcursor;
  private int   size;
  private int   x_incr, y_incr, x_start, x_end, y_start, y_end;
  String letters[] = {"", "a", "b", "c", "d", "e", "f", "g", "h", "i",
                      "j", "k", "l", "m", "n", "o", "p", "q",
                      "r", "s"};
  String numbers[] = {"", "a", "b", "c", "d", "e", "f", "g", "h", "i",
                      "j", "k", "l", "m", "n", "o", "p", "q",
                      "r", "s"};
  String blackfile = "blackstone.gif";
  String whitefile = "whitestone.gif";
  String backgroundfile = "wood.jpg";
  Image blackstoneimage;
  Image whitestoneimage;
  Image backgroundimage;
  Image tempimage;
  Color bk_orangecolor, bk_marooncolor;
  public GoPoint previous_point = null;

  /**  Constructor */

  public GoCanvas(GoBoard board, GoGame game)
  {
    this.board = board;
    this.game = game;
    this.size(board);

    bk_orangecolor = new Color(255, 207, 0);
    bk_marooncolor = new Color(160, 0, 0);

    defaultcursor = new Cursor(Cursor.DEFAULT_CURSOR);
    handcursor = new Cursor(Cursor.HAND_CURSOR);

    setBackground(bk_marooncolor);

    // Load up the stone jpegs into our Image objects
    if (game.applet_p) { // if applet, use applet.getImage(URL)
      URL baseurl = game.parent_applet.getCodeBase();
      System.out.println("getCodeBase returned: " + baseurl.toString()); 
      blackstoneimage = game.parent_applet.getImage(baseurl, blackfile);
      whitestoneimage = game.parent_applet.getImage(baseurl, whitefile);
      //backgroundimage = game.parent_applet.getImage(baseurl, backgroundfile);
    }
    else {  // if application, read file from disk
      blackstoneimage = Toolkit.getDefaultToolkit().getImage(blackfile);
      whitestoneimage = Toolkit.getDefaultToolkit().getImage(whitefile);
      //backgroundimage = Toolkit.getDefaultToolkit().getImage(backgroundfile);
    }

    // Java 1.1 Event Model:  

    //   First, create a MouseAdapter and override the function we care about
    //   (we're using "inner classes" here, another Java 1.1 feature)    

    this.mousey = new MouseAdapter() { 
      public void mouseReleased(MouseEvent e) {
	handle_action(e.getX(), e.getY());
      }
      public void mouseEntered(MouseEvent e) {
	setCursor(handcursor);
      }
      public void mouseExited(MouseEvent e) {
	setCursor(defaultcursor);
      }
    };

    //  Second, make the MouseAdapter object an official "listener"
    //  for GoCanvas's mouse events.  

    this.addMouseListener(mousey);
  }

  private void size(GoBoard board) {
    this.size = board.size();
  }

  /** These two convert from Go-board point to pixel and vice_versa. */

  private Point to_pixel(int x, int y)
  {
    int ret_x, ret_y;

    // Note we decrement x, y due to 1-based internal indexing 
    ret_x = (((x - 1) * x_incr) + x_start);
    ret_y = (((y - 1) * y_incr) + y_start);

    // Restrict it to be within the boundaries of the board... though
    // I don't see why it would ever be an issue, frankly.
    if (ret_x < x_start)  ret_x = x_start;
    if (ret_x > x_end)    ret_x = x_end;
    if (ret_y < y_start)  ret_y = y_start;
    if (ret_y > y_end)    ret_y = y_end;

    return new Point(ret_x, ret_y);
  }

  private Point from_pixel(int x, int y)
  {
    int x_mod, y_mod;

    // game.debug("Mouse event at " + x + ", " + y + ".");

    // Calculate which side is nearer before trimming the garbage.
    x -= x_start;
    y -= y_start;
    x_mod = (x % x_incr);
    y_mod = (y % y_incr);

    if (x_mod > (x_incr / 2))
      x += (x_incr - x_mod);
    else
      x -= x_mod;

    if (y_mod > (y_incr / 2))
      y += (y_incr - y_mod);
    else
      y -= y_mod;

    // Now we are ready to translate to grid points.  Divide by
    // pixel/point ratio, then decrement due to 1-based internal
    // indexing convention.
    x /= x_incr; 
    y /= y_incr; 
    y += 1;
    x += 1;

    game.debug("from_pixel: translates to: " + x + ", " + y + ".");

    return new Point(x, y);
  }

  /** Standard redraw interface for an AWT component: actually draws
      board. */

  public void paint(Graphics g)
  {
    GoPoint pt;
    Rectangle r = getBounds();
    int height, width;

    // Always set our personal "size" to that of the GoBoard we're drawing!
    size(board);

    super.paint(g);

    g.setFont(font);

    // Insure our grid is truly square.  todo: where is abs() ??
    if (r.height < r.width)
    {
      height = r.height;
      width  = r.height;
    }
    else if (r.width < r.height)
    {
      height = r.width;
      width  = r.width;
    }
    else
    {
      height = r.height;
      width  = r.width;
    }

    /* Make sure there's a margin around the playing grid, of course. */
    x_incr  = width  / (size + 1);
    y_incr  = height / (size + 1);
    x_start = x_incr;
    x_end   = x_start + (x_incr * (size - 1));
    y_start = y_incr;
    y_end   = y_start + (y_incr * (size - 1));
    
    // Give it a wooden background.
     g.setColor(bk_orangecolor);
     g.fillRect(0, 0, width, height);
    //    g.drawImage(backgroundimage, 0, 0, width, height, this);

    // Give it border lines.
    g.setColor(Color.red);
    for (int i = 0; i < 3; i++)
      g.drawLine(0, i, width, i);
    for (int i = height - 3; i < height; i++)
      g.drawLine(0, i, width, i);
    for (int i = 0; i < 3; i++)
      g.drawLine(i, 0, i, height);
    for (int i = width - 3; i < width; i++)
      g.drawLine(i, 0, i, height);

    // Draw a black grid on the playing area.
    g.setColor(Color.black);
    for (int i = 1; i <= size; i++)
    {
      g.drawLine(x_start, i * y_incr, x_end, i * y_incr);
      g.drawLine(i * x_incr, y_start, i * x_incr, y_end);
    }

    // Draw whatever stones are on the board
    
    for (int i = 1; i <= this.board.size(); i++)
      for (int j = 1; j <= this.board.size(); j++)
      {
        int color;
        Point spot = new Point(i, j);

        pt = this.board.get_point(spot);
        Point pix = to_pixel(i, j);
          
        if (pt.color_is(GoPoint.BLACK)) 
	  tempimage = blackstoneimage;

        else if (pt.color_is(GoPoint.WHITE)) 
	  tempimage = whitestoneimage;
	
        else if (pt.color_is(GoPoint.EMPTY)) {
          if (pt.handicap()) {
            g.setColor(Color.black);
            // Draw a handicap point (note: point, not stone):
            g.fillOval ((pix.x - (x_incr / 3)),
                        (pix.y - (y_incr / 3)),
                        x_incr / 2,
                        y_incr / 2);
          }
          continue; // skip the stone-drawing below
        }
          
        // Draw a stone of whatever color got set above:
	g.drawImage (tempimage,
		     (pix.x - ((x_incr / 2) - 1)),
		     (pix.y - ((y_incr / 2) - 1)),
		     x_incr - 2,
		     y_incr - 2,
		     this);
   
        // Draw a red dot on the last piece played.
        if (pt == board.last_point()) {
          g.setColor(Color.red);
          g.fillOval (pix.x - (x_incr / 8),
                      pix.y - (y_incr / 8),
                      x_incr/4,
                      y_incr/4);
        }
      }

    // Added by Ben:  label the grid if checkbox is checked
    if ((size == 19) && (game.menu.gridlabels)) {
      g.setColor(Color.red);
      g.setFont(new Font("SansSerif", Font.PLAIN, 8));
      for (int i = 1; i <= 19; i++) {
	g.drawString(letters[i],
		     ((x_start - 2) + (x_incr * (i - 1))),
		     (y_start - 2)
		     );
      } 
      for (int k = 1; k <= 19; k++) {
	g.drawString(numbers[k],
		     (x_start - 12),
		     ((y_start + 5) + (y_incr * (k - 1)))
		     );
      }  
    } 
  }


  /** Function to draw a single point on the board, either black,
     white, or empty.  This gives us the freedom not to have to call
     repaint() on the whole board constantly. */

  public void draw_point(GoPoint pt)
  {
    Graphics g = this.getGraphics();

    Point pix = to_pixel(pt.x, pt.y);
    
    // Drawing an "empty square" is complex, since it looks different
    // if you're on the edge of the board!

    if (pt.color_is(GoPoint.EMPTY))
    {
      int size = board.size();
      // First, give us an orange box
      g.setColor(bk_orangecolor);
      g.fillRect((pix.x - ((x_incr / 2) - 1)),
		(pix.y - ((y_incr / 2) - 1)),
		 x_incr, y_incr);
      
      // Now, draw four lines, provided *each* line passes a logical test.
      g.setColor(Color.black);
      
      if (pt.y != 1)
	g.drawLine(pix.x, (pix.y - (y_incr /2)), pix.x, pix.y);
      if (pt.y != size)
	g.drawLine(pix.x, pix.y, pix.x, (pix.y + (y_incr /2)));
      if (pt.x != 1)
	g.drawLine((pix.x - (x_incr /2)), pix.y, pix.x, pix.y);
      if (pt.x != size)
	g.drawLine(pix.x, pix.y, (pix.x + (x_incr /2)), pix.y);
    }

    else {  // Time to draw a real stone.

      // First, draw the *previous* stone *without* a red dot:
      if ((previous_point != null) &&
	  (! previous_point.color_is(GoPoint.EMPTY)))
      {
	Point pix2 = to_pixel(previous_point.x, previous_point.y);
	if (previous_point.color_is(GoPoint.BLACK))
	  tempimage = blackstoneimage;
	else if (previous_point.color_is(GoPoint.WHITE))
	  tempimage = whitestoneimage;
	g.drawImage (tempimage, (pix2.x - ((x_incr / 2) - 1)),
		     (pix2.y - ((y_incr / 2) - 1)), x_incr - 1, 
		     y_incr - 1, this);
      }

      // Then draw the *new* stone *with* a red dot:
      if (pt.color_is(GoPoint.BLACK))
	tempimage = blackstoneimage;
      else if (pt.color_is(GoPoint.WHITE))
	tempimage = whitestoneimage;
      g.drawImage (tempimage, (pix.x - ((x_incr / 2) - 1)),
		   (pix.y - ((y_incr / 2) - 1)), x_incr - 1, y_incr - 1, this);
      g.setColor(Color.red);
      g.fillOval (pix.x - (x_incr / 8), pix.y - (y_incr / 8),
		    x_incr/4, y_incr/4);
      previous_point = pt;  // and this stone becomes the "previous" one
    }
  }


  /** Instead of redrawing the entire board, loop through our GoPoints
      and look for differences between previous and current colors;
      plot only the GoPoints that have changed. (And then mark them as
      unchanged!) */

  public void update_display()
  {
    GoPoint testpoint = null;
    int size = board.size();

    for (int i = 1; i <= size; i++)
      for (int j = 1; j<= size ; j++) 
      {
	testpoint = board.get_point(i, j);
	if (testpoint.color() != testpoint.previous_color()) 
	{
	  // draw the new point
	  draw_point(testpoint);
	  // and make its previous_color = current_color
	  testpoint.previous_color(testpoint.color()); 
	}
      }
  }



  private boolean check_bounds(int x, int y)
  {
    if (x > x_end) return false;
    if (y > y_end) return false;    
    if (y < y_start) return false;
    if (x < x_start) return false;
    else return true;
  }

  /** Main event handler for the GoCanvas -- the listener receives
      only mousedown events, but this controls the flow of "turns"!  */

  // Mouse events on the canvas are automatically blocked if it's
  // not your turn; it all goes back to the "color" you were
  // assigned at GoGame startup.  Hence observers are automatically
  // blocked!

  public void handle_action(int x, int y)
  {
    Point movepoint;

    if (game.my_turn_p()) {  // proceed only if it's your turn to move

      if (check_bounds(x, y)) { // proceed only if click is within bounds

	movepoint = from_pixel(x, y);   // convert pixel coords to Point 

	// In the two-player version, we once made the move here.  In
	// the observer version, we send a message out that we made a
	// move, wait for the message to propogate back to us via the
	// MessageServer -- then the *GoGame* will handle the message
	// and actually make the real move.

        game.mp.send_message("Place: " + movepoint.x + "," + movepoint.y);
      }
    }

    else {
      if (game.player.color == 0) {
        game.controls.DisplayMessage("You're only observing.");
      }
      else
        game.controls.DisplayMessage("It's not your turn!\n");
    }
  }



}



//////////////////////////////////////////////////////////
// Don't think, just hit `y':
//
// local variables:
// eval: (load-file "bkgomagic.el")
// end:
//////////////////////////////////////////////////////////
