
/** GoDialog: a reinvention of a Swing wheel.  
    
    Simple encapsulated class to handle creation and callbacks of
    dialog boxes.

    I can't believe AWT lacks this!

    API Usage:

    1.  Call constructor with one of three types: INFORMATIONAL,
        OK_CANCEL, INPUT, or PASSWORD.

	GoDialog mydialog(GoDialog.INPUT, "Dialog Title");

    2.  Call the proper display routine, based on dialog type:

        mydialog.showInputDialog("How old are you anyway?", "37 years");

    3.  The display routine always returns a string, equal to "OK",
        "Cancel", or the value in text field, if one was used.  A return
        value of null indicates an error.

  */


import java.awt.*;
import java.awt.event.*;



public class GoDialog extends Dialog implements ActionListener
{
  int type;
  String return_string;

  /** The different types of dialog boxes */
  public static final int INFORMATIONAL = 0;
  public static final int OK_CANCEL = 1;
  public static final int INPUT = 2;
  public static final int PASSWORD = 3;


  GridBagLayout gridbag;
  GridBagConstraints c;
  Button OKbutton, Cancelbutton;
  Label mylabel;
  TextField mytext;
 
  /** The constructor preps things, but doesn't display anything */

  public GoDialog(int type, String title) 
  {
    this.super(new Frame(), true);  // always a modal dialog
    this.type = type;
    this.setTitle("BKGo: " + title);

    gridbag = new GridBagLayout();
    c = new GridBagConstraints();
    this.setLayout(gridbag);
    OKbutton = new Button("OK");
    OKbutton.setActionCommand("OK");
    OKbutton.addActionListener(this);
    Cancelbutton = new Button("Cancel");
    Cancelbutton.setActionCommand("Cancel");
    Cancelbutton.addActionListener(this);
    
  }


  /** Finish building and laying out the window, and display. */

  public String showInformationalDialog(String message)
  {
    if (type != INFORMATIONAL) return null;

    mylabel = new Label(message);

    c.gridx = 0; c.gridy = 0;
    gridbag.setConstraints(mylabel, c);
    this.add(mylabel);
    Panel temppanel = new Panel();
    temppanel.add(OKbutton);
    c.gridx = 0; c.gridy = 1;
    gridbag.setConstraints(temppanel, c);
    this.add(temppanel);
    
    // Finally, display the dialog box
    this.pack();
    this.show();


    // show() blocks until window is dismissed() below
    return return_string;
  }


  /** Finish building and laying out the window, and display. */

  public String showOK_CancelDialog(String message)
  {
    if (type != OK_CANCEL) return null;

    mylabel = new Label(message);

    c.gridx = 0; c.gridy = 0;
    gridbag.setConstraints(mylabel, c);
    this.add(mylabel);
    Panel temppanel = new Panel();
    temppanel.add(OKbutton);
    temppanel.add(Cancelbutton);
    c.gridx = 0; c.gridy = 1;
    gridbag.setConstraints(temppanel, c);
    this.add(temppanel);
    
    // Finally, display the dialog box
    this.pack();
    this.show();
    
    // show() blocks until window is dismissed() below
    return return_string;
  }


  /** Finish building and laying out the window, and display. */

  public String showInputDialog(String message, String defaulttext)
  {
    if (type != INPUT) return null;

    mylabel = new Label(message);
    mytext = new TextField(defaulttext, 15);
    mytext.addActionListener(this);

    c.gridx = 0; c.gridy = 0;
    gridbag.setConstraints(mylabel, c);
    this.add(mylabel);
    c.gridx = 0; c.gridy = 1;
    gridbag.setConstraints(mytext, c);
    this.add(mytext);
    Panel temppanel = new Panel();
    temppanel.add(OKbutton);
    temppanel.add(Cancelbutton);
    c.gridx = 0; c.gridy = 2;
    gridbag.setConstraints(temppanel, c);
    this.add(temppanel);
    
    // Finally, display the dialog box
    this.pack();
    this.show();
    
    // show() blocks until window is dismissed() below
    return return_string;
  }

  /** Finish building and laying out the window, and display. */

  public String showPasswordDialog(String message)
  {
    if (type != PASSWORD) return null;

    mylabel = new Label(message);
    mytext = new TextField(15);
    mytext.setEchoChar('*');
    mytext.addActionListener(this);

    c.gridx = 0; c.gridy = 0;
    gridbag.setConstraints(mylabel, c);
    this.add(mylabel);
    c.gridx = 0; c.gridy = 1;
    gridbag.setConstraints(mytext, c);
    this.add(mytext);
    Panel temppanel = new Panel();
    temppanel.add(OKbutton);
    c.gridx = 0; c.gridy = 2;
    gridbag.setConstraints(temppanel, c);
    this.add(temppanel);
    
    // Finally, display the dialog box
    this.pack();
    this.show();

    
    // show() blocks until window is dismissed() below
    return return_string;
  }


  /** How to handle button events (ActionListener Interface) */

  public void actionPerformed(ActionEvent e)
  {
    if (e.getActionCommand().equals("Cancel")) 
    {
      return_string = "Cancel";
      dismiss();
    }
    else // else actioncommand was either "OK" or the contents of textfield
    {
      if ((type == INFORMATIONAL) || (type == OK_CANCEL)) {
	return_string = "OK";
	dismiss();
      }
      else if ((type == INPUT) || (type == PASSWORD)) {
	return_string = mytext.getText();
	dismiss();
      }
    }

    
  }


  /** Dispose of self neatly -- undoes the modalism */

  public void dismiss() 
  {
    this.setVisible(false);
    this.dispose();
  }


}




//////////////////////////////////////////////////////////
// Don't think, just hit `y':
//
// local variables:
// eval: (load-file "bkgomagic.el")
// end:
//////////////////////////////////////////////////////////
