/**  The menu-bar display at the top of a GoGame window.  */


import java.io.*;
import java.awt.*;
import java.awt.event.*;


public class GoMenu extends MenuBar implements ActionListener
{
  GoGame game;

  Frame aboutbox;
  FilenameFilter sgf_filter;
  String default_filename = "bkgo1.sgf";
  public boolean gridlabels = false;
  MenuItem griditem, debugitem, playeritem, SGFitem;

  String[] fileitems = new String[] {"New Board", "Open Board...", 
				     "Save to Disk...", 
				     "Print...", "Quit"};
  String[] actionitems = new String[] {"Pass", "Redraw", "Name server..."};
  String[] helpitems = new String[] {"About..."};


  /** Le Constructor:  Assemble all menus and build an "about" box too. */

  public GoMenu(GoGame game) {
    this.game = game;

    Menu fileMenu = new Menu("File");
    Menu actionMenu = new Menu("Action");
    Menu helpMenu = new Menu("Help");
    Menu viewMenu = new Menu("View");

    // Assemble the file menu
    for (int i=0; i < fileitems.length; i++) {
      MenuItem item = new MenuItem(fileitems[i]);
      item.addActionListener(this); // listen to own events
      fileMenu.add(item);
    }	
    // Assemble the action menu
    for (int i=0; i < actionitems.length; i++) {
      MenuItem item = new MenuItem(actionitems[i]);
      item.addActionListener(this); // listen to own events
      actionMenu.add(item);
    }	
    // Assemble the view menu more carefully -- MenuItems are class fields!
    playeritem = new MenuItem("Show player window");
    playeritem.addActionListener(this);
    viewMenu.add(playeritem);
    debugitem = new MenuItem("Show debug window");
    debugitem.addActionListener(this);
    viewMenu.add(debugitem);
    SGFitem = new MenuItem("Show SGF window");
    SGFitem.addActionListener(this);
    viewMenu.add(SGFitem);
    griditem = new MenuItem("Show grid labels");
    griditem.addActionListener(this);
    viewMenu.add(griditem);


    // Assemble the help menu
    for (int i=0; i < helpitems.length; i++) {
      MenuItem item = new MenuItem(helpitems[i]);
      item.addActionListener(this); // listen to own events
      helpMenu.add(item);
    }	

    // Add menus to this class, our menubar.
    add(fileMenu);
    add(actionMenu);
    add(viewMenu);
    setHelpMenu(helpMenu);


    // Create a FilenameFilter to show only *.sgf files in load/save

    // BY THE WAY, THIS DOESN'T WORK -- Sun acknowledges that the while
    // the idea behind this call is good, they *admit* that it has never
    // worked -- and never *will* work under vanilla AWT.  Because the
    // AWT FileDialog maps to native GUI calls, they don't have this
    // sort of control for some reason.  This is why they wrote their
    // own standard JFileDialog in Swing -- filters work there!

    // Thus, we'll keep this around, until we update to Swing someday.
  
    sgf_filter = new FilenameFilter() {
      public boolean accept(File dir, String name) {
	int i = name.lastIndexOf('.');
	if (i > 0 &&  i < name.length() - 1) {
          String extension = name.substring(i+1).toLowerCase();
	  System.out.println("Found extension: " + extension);
          if (extension.equals("sgf")) { return true; }
          else { return false; }
	  }
	return false;
      }
    };
    

    // Epilogue:  create a decent looking "about" box
    
    aboutbox = new Frame("About BKGo");
    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    aboutbox.setLayout(gridbag);
    Label label1 = new Label("BKGo:  A Java 1.1 networked Go game, version "
			     + game.version.version_number);
    label1.setFont(new Font("Serif", Font.BOLD, 14));
    c.gridx = 0; c.gridy = 0;
    gridbag.setConstraints(label1, c);
    aboutbox.add(label1);
    Label label2 = new Label("by Karl Fogel <kfogel@red-bean.com>");
    c.gridx = 0; c.gridy = 1;
    gridbag.setConstraints(label2, c);
    aboutbox.add(label2);
    Label label3 = new Label("and Ben Collins-Sussman <sussman@red-bean.com>");
    c.gridx = 0; c.gridy = 2;
    gridbag.setConstraints(label3, c);
    aboutbox.add(label3);
    Label label4 = new Label("for info, see http://www.red-bean.com/bkgo");
    c.gridx = 0; c.gridy = 3;
    gridbag.setConstraints(label4, c);
    aboutbox.add(label4);
    aboutbox.pack();
    aboutbox.addWindowListener( new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        aboutbox.setVisible(false);
      }
    });
    
  }
    

  /**  ActionListener interface: called on each menu event */

  public void actionPerformed(ActionEvent e) {
    String menucommand = e.getActionCommand();

    game.debug("Menu Command is '" + menucommand + "'.");

    if (menucommand.equals("Redraw")) {
      game.canvas.repaint();
      game.controls.DisplayMessage("Board redrawn.");
    }

    else if (menucommand.equals("New Board")) {
      if (game.player.color == GoPlayer.OBSERVER) 
      {
	GoDialog dlg = new GoDialog(GoDialog.INFORMATIONAL, "Sorry");
	dlg.showInformationalDialog("Observers can't change the game.");
      }

      else
      {
	GoDialog dialog1 = new GoDialog(GoDialog.OK_CANCEL, "New Game");
	String reply = dialog1.showOK_CancelDialog
	  ("Are you sure you want to start a new game?");
	
	if (reply.equals("OK")) {
	  GoDialog dialog2 = new GoDialog(GoDialog.INPUT, "Board Size");
	  String requested_size = dialog2.showInputDialog
	    ("How large is the new board?", "19");
	  if (requested_size.equals(null)) { requested_size = "19"; }
	  int new_size = Integer.parseInt(requested_size);    
	  
	  // create a new GoBoard for this client
	  game.debug("Creating new board, size " + new_size);
	  game.board = new GoBoard(game, new_size);
	  
	  // formally request the server to use this client's GoBoard
	  game.mp.send_message("NewBoard");
	}
      }
	
    }

    else if (menucommand.equals("Open Board...")) {
      if (game.applet_p) {
	GoDialog dialog5 = new GoDialog(GoDialog.INFORMATIONAL, "Sorry");
	dialog5.showInformationalDialog("Sorry, an applet can't access disk.");
      }
      else if (game.player.color == GoPlayer.OBSERVER) 
      {
	GoDialog dlg = new GoDialog(GoDialog.INFORMATIONAL, "Sorry");
	dlg.showInformationalDialog("Observers can't change the game.");
      }
      else {
	FileDialog opendialog = new FileDialog(new Frame(),
					       "BKGo:  Open an SGF File",
					       FileDialog.LOAD);
	opendialog.setFilenameFilter(sgf_filter); // show only *.sgf files
	opendialog.setFile(default_filename);
	opendialog.show();  // blocks till a file is chosen
	String filename = opendialog.getFile();  // get the file chosen

	// Open the file and send the SGF stream to the server.
	game.open_board(filename);
      }
    }

    else if (menucommand.equals("Save to Disk...")) {
      // try to save to disk if we're a real application.
      if (game.applet_p) {
	GoDialog dialog5 = new GoDialog(GoDialog.INFORMATIONAL, "Sorry");
	dialog5.showInformationalDialog("Sorry, an applet can't access disk.");
      }
      else {
	FileDialog savedialog = new FileDialog(new Frame(), 
					       "BKGo: Save to SGF File",
					       FileDialog.SAVE);
	savedialog.setFilenameFilter(sgf_filter); // show only *.sgf files
	savedialog.setFile(default_filename);
	savedialog.show();  // blocks till a file is chosen
	String fname = savedialog.getFile();  // get the file chosen
	default_filename = fname; // save it for next time too
	SGFnode root = (SGFnode) game.board.GameTree;
	root.set_prop_literal("GN", fname);  // place filename in GameTree!
	System.out.println("File chosen: " + fname);

	if (fname == null) {
	  game.controls.DisplayMessage("Save game cancelled.");
	}
	else {
	  game.save_board(fname);
	  game.controls.DisplayMessage("Game saved as " + fname);
	}
      }
    }

    else if (menucommand.equals("Print...")) {
      if (game.applet_p) {
	GoDialog gd = new GoDialog(GoDialog.INFORMATIONAL, "Sorry");
	gd.showInformationalDialog("Sorry, an applet can't print.");
      }
      else {
	game.print_window();
      }
    }

    else if (menucommand.equals("Pass")) {
      if (game.my_turn_p()) {
        game.controls.DisplayMessage("You pass.");
        game.mp.send_message("Pass");
      }
      else
        game.controls.DisplayMessage("It's not your turn!");
    }

    else if (menucommand.equals("Quit")) {
      game.quit();
    }

    else if (menucommand.equals("About...")) {
      aboutbox.setVisible(true);
    }

    else if (menucommand.equals("Show grid labels")) {
      gridlabels = true;
      griditem.setLabel("Hide grid labels");
      game.canvas.repaint();
    }

    else if (menucommand.equals("Hide grid labels")) {
      gridlabels = false;
      griditem.setLabel("Show grid labels");
      game.canvas.repaint();
    }

    else if (menucommand.equals("Show debug window")) {
      game.debugframe.setVisible(true);
      debugitem.setLabel("Hide debug window");
    }

    else if (menucommand.equals("Hide debug window")) {
      game.debugframe.setVisible(false);
      debugitem.setLabel("Show debug window");
    }

    else if (menucommand.equals("Show SGF window")) {
      game.SGFframe.setVisible(true);
      game.display_tree();
      SGFitem.setLabel("Hide SGF window");
    }

    else if (menucommand.equals("Hide SGF window")) {
      game.SGFframe.setVisible(false);
      SGFitem.setLabel("Show SGF window");
    }

    else if (menucommand.equals("Show player window")) {
      
      game.userframe.setVisible(true);
      playeritem.setLabel("Hide player window");
    }

    else if (menucommand.equals("Hide player window")) {
      game.userframe.setVisible(false);
      playeritem.setLabel("Show player window");
    }

    else if (menucommand.equals("Name server...")) {
      if (game.player.color == GoPlayer.OBSERVER) 
      {
	GoDialog dlg = new GoDialog(GoDialog.INFORMATIONAL, "Sorry");
	dlg.showInformationalDialog("Observers can't name the server.");
      }
      else {
	GoDialog gd1 = new GoDialog(GoDialog.INPUT, "Choose Name");
	String name = gd1.showInputDialog("Set server name:", "no_name");
	if ((name == "Cancel") || (name == null)) {
	  // do nothing.
	}
	else {
	  game.mp.send_message("SetName: " + name);
	}
      }
    }

  }

}



//////////////////////////////////////////////////////////
// Don't think, just hit `y':
//
// local variables:
// eval: (load-file "bkgomagic.el")
// end:
//////////////////////////////////////////////////////////

