
/**  A simple class (structure) to represent a player. */

import java.io.*;
import java.net.*;

public class GoPlayer implements Serializable
{
  int IDnum;
  int color;

  String name;

  public String DNSname;
  public String OS;
  public String OS_ver;

  ClientConnector CC;

  public static final int OBSERVER = 0;
  public static final int BLACK = 1;
  public static final int WHITE = 2;
    
  /** Constructor is called by bkgo; fills in everything except ID and CC.*/

  public GoPlayer(String name, int color, boolean applet_p) 
  {
    this.name = name;
    this.color = color;

    if (applet_p) {
      DNSname="web browser";
    }
    else {
      try {
	InetAddress My_Address = InetAddress.getLocalHost();
	DNSname = My_Address.getHostName();
      }
      catch (UnknownHostException e) {
	System.out.println(e);
	DNSname = "unknown DNS name";
      }
    }  
    OS = System.getProperty("os.name");
    OS_ver = System.getProperty("os.version");
    
  }

  
  /** return a terse string for submitting over a socket:

         a     b    c       d      e
      int:int  :str :str    :str   :str
      UID:color:name:DNSname:OSname:OSver
  */

  public String toString()
  {
    StringWriter sw = new StringWriter();

    sw.write(String.valueOf(IDnum));
    sw.write(":" + String.valueOf(color));
    sw.write(":" + name);
    sw.write(":" + DNSname);
    sw.write(":" + OS);
    sw.write(":" + OS_ver);

    sw.flush();
    sw.close();
    return sw.toString();
  }


  /** parse a terse string and fill in my fields appropriately */

  public void parseString(String fields)
  {
    int a, b, c, d, e;

    //    System.out.println("Attempting to parse '" + fields + "'");

    a = fields.indexOf(':');
    IDnum = Integer.parseInt(fields.substring(0, a));

    b = fields.indexOf(':', a+1);  // search forward from point 'a'
    color = Integer.parseInt(fields.substring(a+1, b));

    c = fields.indexOf(':', b+1); 
    name = fields.substring(b+1, c);

    d = fields.indexOf(':', c+1); 
    DNSname = fields.substring(c+1, d);

    e = fields.indexOf(':', d+1); 
    OS = fields.substring(d+1, e);

    OS_ver = fields.substring(e+1);

    //System.out.println("My new pretty-string is " + this.toPrettyString());
  }


  /** return a nicer-looking string to display in player-list */ 
 
  public String toPrettyString()
  {
    StringWriter sw = new StringWriter();
    String colstring;
    if (color == BLACK) { colstring = "black"; }
    else if (color == WHITE) { colstring = "white"; }
    else if (color == OBSERVER) { colstring = "observer"; }
    else { colstring = "unknown color"; }

    sw.write("ID ");
    sw.write(String.valueOf(IDnum));
    sw.write(": " + name + " (" + colstring + "), on ");
    sw.write(DNSname + " (" + OS + " " + OS_ver + ")");
    sw.flush();
    sw.close();
    return sw.toString();
  }

}


//////////////////////////////////////////////////////////
// Don't think, just hit `y':
//
// local variables:
// eval: (load-file "bkgomagic.el")
// end:
//////////////////////////////////////////////////////////
