/** GoPoint extends Point and represents a point on the board. */


import java.io.*;
import java.awt.*;
import java.applet.*;
import java.util.Vector;
import java.util.Enumeration;


class GoPoint extends Point implements Serializable, Cloneable
{
  // All points have a "color" (i.e., state):
  public static final int EMPTY = 0;
  public static final int BLACK = 1;
  public static final int WHITE = 2;

  /** Extending Point means we have x, y, Point(x,y), move(x,y), and
      equals(Obj).  Also translate(dx,dy), but we don't use that yet.  */

  GoBoard board; 
  private int color; 
  private int previous_color = EMPTY;
  private boolean handicap = false;
  int group_id; // Which group is this a member of?


  // Constructor
  public GoPoint (GoBoard gboard, int x, int y, int color, int gid)
  {
    super(x, y);
    this.board = gboard;
    this.color(color);
    this.gid(gid);

    /* Discover if this is a handicap point.
       todo: handle various board sizes below */

    int size = board.size();
    if (size == 19) {
      if (((x == 4) && (y == 4))    ||
          ((x == 4) && (y == 10))   ||
          ((x == 4) && (y == 16))   ||
          ((x == 10) && (y == 4))   ||
          ((x == 10) && (y == 10))  ||
          ((x == 10) && (y == 16))  ||
          ((x == 16) && (y == 4))   ||
          ((x == 16) && (y == 10))  ||
          ((x == 16) && (y == 16))) {
        this.handicap = true;
      }
    }
  }

  /**  Return the other color, wouldn't'cha know? */
 
  public static int other_color(int color) {
    return ((color % 2) + 1);
  }

  public int gid() {
    return group_id;
  }

  public void gid(int id) {
    group_id = id;
  }

  public boolean gid_is(int gid) {
    return (this.gid() == gid);
  }

  public int color() {
    return color;
  }

  public int previous_color() {
    return previous_color;
  }

  public void color(int newcolor) {
    previous_color = color;
    color = newcolor;
  }

  public void previous_color(int newcolor) {
    previous_color = newcolor;
  }

  public boolean color_is(int color) {
    return (this.color == color);
  }

  /** Handicap can be publicly retrieved, but not set.  Only
	constructor sets it, based on Board size. */

  public boolean handicap() {
    return this.handicap;
  }

  public void move(int x, int y) {
    // Do nothing.  Overrided because GoPoints can't be moved right now.
  }

  public void place_stone(int color) {
    this.color(color);
  }

  public void remove_stone() {
    color = EMPTY; 

    /*    // Added by Ben:
	  if (this.color_is(BLACK)) board.whitescore++;
	  if (this.color_is(WHITE)) board.blackscore++;    */
  }

  /**  Returns the number of liberties for this point. */

  public int liberties() {
    return this.board.liberties(this);
  }

  public Object clone() 
  {
    try {
      return super.clone();
    } 
    catch (CloneNotSupportedException e) {
      throw new InternalError(e.getMessage());
    }
  }
}


//////////////////////////////////////////////////////////
// Don't think, just hit `y':
//
// local variables:
// eval: (load-file "bkgomagic.el")
// end:
//////////////////////////////////////////////////////////
