
/** A panel for displaying the number of captured stones
    on each side (and whose turn it is). */


import java.awt.*;
import java.awt.event.*;


public class ScorePanel extends Canvas
{
  GoGame game;
  FontMetrics metrics;
  int ascent;
  Image whitestone;
  Image blackstone;

  public ScorePanel(GoGame game) 
  {
    this.game = game;

    setFont(game.bk_font);               // standard bk_font 
    metrics = this.getFontMetrics(game.bk_font);
    ascent = metrics.getAscent();

    setSize(400, ascent*2);  // make it about as long as the GoGame frame
    
    // use same transparent stone image that GoCanvas is using...
    whitestone = game.canvas.whitestoneimage;
    blackstone = game.canvas.blackstoneimage;
  }

  public void paint(Graphics g)
  {
    int widthpointer;                    // how many pixels have we moved?
    setBackground(new Color(160, 0, 0));  // standard bkgo_maroon

    Dimension d = getSize();
    g.setColor(Color.white);
    
    // Get metrics of current font for proper positioning:

    // Get our variables to display
    String movesmade = Integer.toString(game.board.number_of_moves());
    String whitescore = Integer.toString(game.board.white_score());
    String blackscore = Integer.toString(game.board.black_score());

    // Output moves made
    String movestring = "Moves: " + movesmade + ", ";
    g.drawString(movestring, 0, ascent*3/2);
    widthpointer = metrics.stringWidth(movestring);
		 
    // Output black captures
    String blackstring = "Captured by Black: ";
    g.drawString(blackstring, widthpointer, ascent*3/2);
    widthpointer += metrics.stringWidth(blackstring);

    // Output white stone:
    g.drawImage(whitestone, widthpointer, 0, ascent*2, ascent*2, this);

    // Output black score:
    g.setColor(Color.black);
    int boffsetx = metrics.stringWidth(blackscore) / 2;
    g.drawString(blackscore, 
		 widthpointer + ascent - boffsetx, 
		 ascent*3/2);
    g.setColor(Color.white);
    widthpointer += (ascent * 3);


    // Output white captures
    String whitestring = "Captured by White: ";
    g.drawString(whitestring, widthpointer, ascent*3/2);
    widthpointer += metrics.stringWidth(whitestring);

    // Output black stone:
    g.drawImage(blackstone, widthpointer, 0, ascent*2, ascent*2, this);

    // Output black score:
    int woffsetx = metrics.stringWidth(whitescore) / 2;
    g.drawString(whitescore, 
		 widthpointer + ascent - woffsetx, 
		 ascent*3/2);
  }

}



//////////////////////////////////////////////////////////
// Don't think, just hit `y':
//
// local variables:
// eval: (load-file "bkgomagic.el")
// end:
//////////////////////////////////////////////////////////
