
/*  BKGo:  a network Go game written in Java 1.1

    By Karl Fogel <kfogel@red-bean.com>
    and Ben Collins-Sussman <sussman@red-bean.com>

    January 1999

*/

import java.io.*;
import java.awt.*;
import java.awt.event.*;



/**  The bkgo class has two purposes:

     1.  To present a nice startup screen to users (avoiding the necessity
         of clunky command-line arguments)

     2.  Contains the main() function which greates a "GoGame" and
         initializes the network correctly.

*/

public class bkgo extends Frame 
implements ActionListener, ItemListener
{
  GoGame the_game;
  MessageServer ms;
  String network_role;
  String chosen_hostname;
  String chosen_port;
  String chosen_name;
  int chosen_color;
  int port;
  CheckboxGroup gamegroup, colorgroup;
  Checkbox serverbutton, clientbutton, demobutton, listbutton;
  Checkbox blackbutton, whitebutton, observerbutton;
  TextField namefield, sport1, sport2, shost;
  public static final int OBSERVER = 0;
  public static final int BLACK = 1;
  public static final int WHITE = 2;
    
  // Constructor:  builds the startup window GUI

  public bkgo() 
  {
    Panel welcomepanel = new Panel();
    welcomepanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    Label welcomelabel = new Label("Welcome to BKGo.");
    welcomelabel.setForeground(Color.blue);
    welcomepanel.add(welcomelabel);

    serverbutton = new 
      Checkbox("Start a new server (and client) on port");
    clientbutton = new 
      Checkbox("Connect to a server on");
    demobutton = new 
      Checkbox("Demo Mode (play both sides here)");
    listbutton = new 
      Checkbox("Huh?  Just show me a list of games in progress.");
    serverbutton.addItemListener(this);
    clientbutton.addItemListener(this);
    demobutton.addItemListener(this);
    listbutton.addItemListener(this);
    sport1 = new TextField("30000", 5);
    sport2 = new TextField("30000", 5);
    Label plabel = new Label(", port");
    plabel.setForeground(Color.black);
    shost = new TextField("localhost", 12);

    gamegroup = new CheckboxGroup();
    serverbutton.setCheckboxGroup(gamegroup);
    clientbutton.setCheckboxGroup(gamegroup);
    demobutton.setCheckboxGroup(gamegroup);
    listbutton.setCheckboxGroup(gamegroup);
    demobutton.setState(true);

    Panel gamepanel = new Panel();
    //    gamepanel.setBorder(new EtchedBorder());
    Panel panel1 = new Panel();
    panel1.setLayout(new FlowLayout(FlowLayout.LEFT));
    panel1.add(serverbutton);
    panel1.add(sport1);
    Panel panel2 = new Panel();
    panel2.setLayout(new FlowLayout(FlowLayout.LEFT));
    panel2.add(clientbutton);
    panel2.add(shost);
    panel2.add(plabel);
    panel2.add(sport2);
    Panel panel3 = new Panel();
    panel3.setLayout(new FlowLayout(FlowLayout.LEFT));
    panel3.add(demobutton);
    Panel panel4 = new Panel();
    panel4.setLayout(new FlowLayout(FlowLayout.LEFT));
    panel4.add(listbutton);
    gamepanel.setLayout(new GridLayout(0, 1));
    gamepanel.add(panel1);
    gamepanel.add(panel2);
    gamepanel.add(panel3);
    gamepanel.add(panel4);

    blackbutton = new Checkbox("...as black player");
    whitebutton = new Checkbox("...as white player");
    observerbutton = new Checkbox("...as an observer");
    blackbutton.setEnabled(false);
    whitebutton.setEnabled(false);
    observerbutton.setEnabled(false);

    colorgroup = new CheckboxGroup();
    blackbutton.setCheckboxGroup(colorgroup);
    whitebutton.setCheckboxGroup(colorgroup);
    observerbutton.setCheckboxGroup(colorgroup);
    blackbutton.setState(true);

    Panel colorpanel = new Panel();
    //    colorpanel.setBorder(new EtchedBorder());
    colorpanel.setLayout(new GridLayout(0, 1));
    colorpanel.add(blackbutton);
    colorpanel.add(whitebutton);
    colorpanel.add(observerbutton);


    Panel namepanel = new Panel(new FlowLayout(FlowLayout.LEFT));
    Label namelabel = new Label("Name: ");
    namefield = new TextField("anonymous coward", 20);
    namepanel.add(namelabel);
    namepanel.add(namefield);
    //    namepanel.setBorder(new EtchedBorder());

    Panel buttonpanel = new Panel();
    //    buttonpanel.setBorder(new EtchedBorder());
    Button startbutton = new Button("Start");
    Button quitbutton = new Button("Quit");
    buttonpanel.add(startbutton);
    buttonpanel.add(quitbutton);
    startbutton.setActionCommand("start");
    quitbutton.setActionCommand("quit");
    startbutton.addActionListener(this);
    quitbutton.addActionListener(this);


    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    this.setLayout(gridbag);
    
    c.gridx = 0;  c.gridy = 0;
    gridbag.setConstraints(welcomepanel, c);
    this.add(welcomepanel);
    c.gridy = 1;
    gridbag.setConstraints(gamepanel, c);
    this.add(gamepanel);
    c.gridy = 2;
    gridbag.setConstraints(colorpanel, c);
    this.add(colorpanel);
    c.gridy = 3;
    gridbag.setConstraints(namepanel, c);
    this.add(namepanel);
    c.gridy = 4;
    gridbag.setConstraints(buttonpanel, c);
    this.add(buttonpanel);


    this.addWindowListener( new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.out.println
          ("\nThis software is protected by the GPL.");
        System.out.println
          ("See COPYING for details.\n");
        System.exit(0);	   
      }
    });
  }


  /** How to respond to Start/Quit buttons -- (ActionListener interface) */

  public void actionPerformed(ActionEvent e) 
  {
    if (e.getActionCommand().equals("start")) {
      if (!(error_check())) {
        start_game();
        start_network();
      }
    }
    else if (e.getActionCommand().equals("quit")) {
      System.out.println
        ("\nThis software is protected by the GPL.");
      System.out.println
        ("See COPYING for details.\n");
      System.exit(0);
    }
  }

  /** How to respond to radiobuttons -- (ItemListener interface) */

  public void itemStateChanged(ItemEvent e) 
  {
    Checkbox box = gamegroup.getSelectedCheckbox();

    if ((box == demobutton) ||
             (box == listbutton) ) {
      blackbutton.setEnabled(false);
      whitebutton.setEnabled(false);
      observerbutton.setEnabled(false);
    }
    else if ((box == serverbutton) ||
             (box == clientbutton) ) {
      blackbutton.setEnabled(true);
      whitebutton.setEnabled(true);
      observerbutton.setEnabled(true);
    }
  }




  /** Before starting the game, sanity-check all the options! 
      True  --> error was found
      False --> no errors found
  */

  public boolean error_check() 
  {
    // set chosen_name
    chosen_name = namefield.getText();
    if ((chosen_name.equals(null)) || (chosen_name.length() == 0)) {
      String err = "No player name chosen.";
      System.out.println(err);
      return true;
    }
     
    // set chosen_color
    Checkbox boxcolor = colorgroup.getSelectedCheckbox();
    if (boxcolor == blackbutton) { chosen_color = BLACK; }
    else if (boxcolor == whitebutton) { chosen_color = WHITE; }
    else if (boxcolor == observerbutton) { chosen_color = OBSERVER; }
    else {
      String err = "Unknown color chosen.";
      System.out.println(err);
      return true;
    }

    // set variable network_role equal to "SERVER" or "CLIENT", etc.
    Checkbox gamebox = gamegroup.getSelectedCheckbox();
    if (gamebox == serverbutton) { network_role = "SERVER"; }
    else if (gamebox == clientbutton) { network_role = "CLIENT"; }
    else if (gamebox == demobutton) { network_role = "DEMO"; }
    else if (gamebox == listbutton) { network_role = "LIST"; }

    if (network_role.equals(null)) {
      String err = "No network option selected.";
      System.out.println(err);
      return true;
    }

    if ((network_role.equals("SERVER")) ||
        (network_role.equals("DEMO"))) {
      // set chosen_port
      chosen_port = sport1.getText();
      if ((chosen_port.equals(null)) || (chosen_port.length() == 0)) {
        String err = "No port specified.";
        System.out.println(err);
        return true;
      }
    }

    if (network_role.equals("CLIENT")) {
      // set chosen_hostname
      chosen_hostname = shost.getText();
      if ((chosen_hostname.equals(null)) || 
          (chosen_hostname.length() == 0)) {
        String err = "No hostname specified.";
        System.out.println(err);
        return true;
      }
      // set chosen_port
      chosen_port = sport2.getText();
      if ((chosen_port.equals(null)) || (chosen_port.length() == 0)) {
        String err = "No port specified.";
        System.out.println(err);
        return true;
      }
    }

    if (network_role.equals("LIST")) {
      GoDialog mydialog1 = new GoDialog(GoDialog.INFORMATIONAL, "Sorry");
      String val = mydialog1.showInformationalDialog
	("Bkgo list server not yet available.");
      System.out.println("dialog returned value: " + val);
      GoDialog mydialog2 = new GoDialog(GoDialog.INPUT, "Question");
      val = mydialog2.showInputDialog("How old are you anyway?", "26");
      System.out.println("dialog returned value: " + val);
      GoDialog mydialog3 = new GoDialog(GoDialog.OK_CANCEL, "Yah?");
      val = mydialog3.showOK_CancelDialog("May I Continue?");
      System.out.println("dialog returned value: " + val);
      return true;
    }
    // we survived all the error-checking:
    return false;
  }


  /** Then instantiate a full GoGame object */

  public void start_game() 
  {
    String windowtitle;
    if (network_role.equals("DEMO")) { windowtitle = " (demo mode)"; }
    else if (chosen_color == BLACK) { windowtitle = " (playing Black)"; }
    else if (chosen_color == WHITE) { windowtitle = " (playing White)"; }
    else if (chosen_color == OBSERVER) { windowtitle = " (observing)"; }
    else { windowtitle = ": unknown role"; };
    the_game = new GoGame(false, null);// game WITHOUT applet restrictions
    the_game.player = new GoPlayer(chosen_name, chosen_color, false);
    the_game.network_role = network_role;
    the_game.setTitle("BK Go: " + chosen_name + windowtitle);
    the_game.setSize(800, 600); // starting size of window
    the_game.controls.DisplayMessage("Not connected to server.");
  }


  /** Then set up the network objects here */

  public void start_network() 
  {
    try {
      if ((network_role.equals("SERVER"))
          || (network_role.equals("DEMO"))) {  // a server and client

        port = Integer.parseInt(chosen_port);
        the_game.debug("Creating a MessageServer on port " +
                       port + "...\n");
        ms = new MessageServer(port);
	ms.start();
        // listen for connection 
        the_game.debug("Making own MessagePasser client...\n");
        the_game.mp = new 
          MessagePasser(the_game, "localhost", port);
        the_game.mp.start();
      }
	    
      else if (network_role.equals("CLIENT")) {  // just a client
        port = Integer.parseInt(chosen_port);
        // reach out and touch someone
        the_game.mp = new 
          MessagePasser(the_game, chosen_hostname, port);
        the_game.mp.start();
      }
    }
    catch (IOException e) {
      System.err.println("Network Error! " + e);
      if ((network_role.equals("SERVER")) || 
          (network_role.equals("DEMO"))) {
        ms.stop();
      }
      the_game.dispose();
    }
  }



  /** Main:  creates the startup window.  That's it!  */

  public static void main(String args[]) 
  {
    bkgo StartWindow = new bkgo();
    StartWindow.setTitle("Welcome to BKGo!");
    StartWindow.pack();
    StartWindow.setVisible(true);
  }

}



//////////////////////////////////////////////////////////
// Don't think, just hit `y':
//
// local variables:
// eval: (load-file "bkgomagic.el")
// end:
//////////////////////////////////////////////////////////
