
/*  BKGo Applet:  a network Go game written in Java 1.1

    This is the APPLET version of the start-up window for bkgo.
    It explictly leaves out the SERVER/DEMO options, leaving the choice
    of only connecting as a CLIENT.

    By Karl Fogel <kfogel@red-bean.com>
    and Ben Collins-Sussman <sussman@red-bean.com>

    January 1999

*/

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;


/**  The bkgo class has two purposes:

     1.  To present a nice startup screen to users (avoiding the necessity
         of clunky command-line arguments)

     2.  Contains the main() function which greates a "GoGame" and
         initializes the network correctly.

*/

public class bkgo_applet extends Applet 
implements ActionListener, ItemListener
{
  GoGame the_game;
  String network_role;
  String chosen_hostname;
  String chosen_port;
  String chosen_name;
  int chosen_color;
  int port;
  CheckboxGroup gamegroup, colorgroup;
  Checkbox serverbutton, clientbutton, demobutton;
  Checkbox blackbutton, whitebutton, observerbutton;
  TextField namefield, sport1, sport2, shost;
  public static final int OBSERVER = 0;
  public static final int BLACK = 1;
  public static final int WHITE = 2;
    
  // Constructor:  builds the startup window GUI.  
  // Instead of being called bkgo(), applet requires that we call it init()

  public void init() 
  {
    Panel welcomepanel = new Panel();
    welcomepanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    Label welcomelabel = new Label("Welcome to BKGo.");
    welcomelabel.setForeground(Color.blue);
    welcomepanel.add(welcomelabel);

    clientbutton = new 
      Checkbox("Connect to a server on");
    clientbutton.addItemListener(this);
    clientbutton.setState(true);
    String portnum = getParameter("portnum");
    sport1 = new TextField(portnum, 5);
    sport2 = new TextField(portnum, 5);
    Label plabel = new Label(", port");
    plabel.setForeground(Color.black);
    String server = getParameter("servername");
    shost = new TextField(server);
    shost.setEditable(false);
    sport1.setEditable(false);
    sport2.setEditable(false);

    gamegroup = new CheckboxGroup();
    clientbutton.setCheckboxGroup(gamegroup);
    clientbutton.setState(true);

    Panel gamepanel = new Panel();
    Panel panel2 = new Panel();
    panel2.setLayout(new FlowLayout(FlowLayout.LEFT));
    panel2.add(clientbutton);
    panel2.add(shost);
    panel2.add(plabel);
    panel2.add(sport2);
    gamepanel.setLayout(new GridLayout(0, 1));
    gamepanel.add(panel2);

    blackbutton = new Checkbox("...as black player");
    whitebutton = new Checkbox("...as white player");
    observerbutton = new Checkbox("...as an observer");

    colorgroup = new CheckboxGroup();
    blackbutton.setCheckboxGroup(colorgroup);
    whitebutton.setCheckboxGroup(colorgroup);
    observerbutton.setCheckboxGroup(colorgroup);
    blackbutton.setState(true);

    Panel colorpanel = new Panel();
    //    colorpanel.setBorder(new EtchedBorder());
    colorpanel.setLayout(new GridLayout(0, 1));
    colorpanel.add(blackbutton);
    colorpanel.add(whitebutton);
    colorpanel.add(observerbutton);


    Panel namepanel = new Panel(new FlowLayout(FlowLayout.LEFT));
    Label namelabel = new Label("Name: ");
    namefield = new TextField("anonymous coward", 20);
    namepanel.add(namelabel);
    namepanel.add(namefield);
    //    namepanel.setBorder(new EtchedBorder());

    Panel buttonpanel = new Panel();
    //    buttonpanel.setBorder(new EtchedBorder());
    Button startbutton = new Button("Start");
    buttonpanel.add(startbutton);
    startbutton.setActionCommand("start");
    startbutton.addActionListener(this);


    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    this.setLayout(gridbag);
    
    c.gridx = 0;  c.gridy = 0;
    gridbag.setConstraints(welcomepanel, c);
    this.add(welcomepanel);
    c.gridy = 1;
    gridbag.setConstraints(gamepanel, c);
    this.add(gamepanel);
    c.gridy = 2;
    gridbag.setConstraints(colorpanel, c);
    this.add(colorpanel);
    c.gridy = 3;
    gridbag.setConstraints(namepanel, c);
    this.add(namepanel);
    c.gridy = 4;
    gridbag.setConstraints(buttonpanel, c);
    this.add(buttonpanel);

  }


  /** How to respond to Start/Quit buttons -- (ActionListener interface) */

  public void actionPerformed(ActionEvent e) 
  {
    if (e.getActionCommand().equals("start")) {
      if (!(error_check())) {
        start_game();
        start_network();
      }
    }
  }

  /** How to respond to radiobuttons -- (ItemListener interface) */

  public void itemStateChanged(ItemEvent e) 
  {
    Checkbox box = gamegroup.getSelectedCheckbox();

    if (box == clientbutton ) {
      blackbutton.setEnabled(true);
      whitebutton.setEnabled(true);
      observerbutton.setEnabled(true);
    }
  }



  /** Before starting the game, sanity-check all the options! 
      True  --> error was found
      False --> no errors found
  */

  public boolean error_check() 
  {
    // set chosen_name
    chosen_name = namefield.getText();
    if ((chosen_name == null) || (chosen_name.length() == 0)) {
      String err = "No player name chosen.";
      System.out.println(err);
      return true;
    }
     
    // set chosen_color
    Checkbox boxcolor = colorgroup.getSelectedCheckbox();
    if (boxcolor == blackbutton) { chosen_color = BLACK; }
    else if (boxcolor == whitebutton) { chosen_color = WHITE; }
    else if (boxcolor == observerbutton) { chosen_color = OBSERVER; }
    else {
      String err = "Unknown color chosen.";
      System.out.println(err);
      return true;
    }

    // set variable network_role equal to "SERVER" or "CLIENT", etc.
    Checkbox gamebox = gamegroup.getSelectedCheckbox();
    if (gamebox == clientbutton) { network_role = "CLIENT"; }

    if (network_role == null) {
      String err = "No network option selected.";
      System.out.println(err);
      return true;
    }

    if (network_role.equals("CLIENT")) {
      // set chosen_hostname
      chosen_hostname = shost.getText();
      if ((chosen_hostname == null) || 
          (chosen_hostname.length() == 0)) {
        String err = "No hostname specified.";
        System.out.println(err);
        return true;
      }
      // set chosen_port
      chosen_port = sport2.getText();
      if ((chosen_port == null) || (chosen_port.length() == 0)) {
        String err = "No port specified.";
        System.out.println(err);
        return true;
      }
    }


    // we survived all the error-checking:
    return false;
  }


  /** Then instantiate a full GoGame object */

  public void start_game() 
  {
    String windowtitle;
    if (chosen_color == BLACK) { windowtitle = " (playing Black)"; }
    else if (chosen_color == WHITE) { windowtitle = " (playing White)"; }
    else if (chosen_color == OBSERVER) { windowtitle = " (observing)"; }
    else { windowtitle = ": unknown role"; };
    the_game = new GoGame(true, this); //start a game WITH applet restrictions
    the_game.player = new GoPlayer(chosen_name, chosen_color, true);
    the_game.network_role = network_role;
    the_game.setTitle("BK Go: " + chosen_name + windowtitle);
    the_game.setSize(800, 600); // starting size of window
    the_game.controls.DisplayMessage("Not connected to server.");
  }


  /** Then set up the network objects here */

  public void start_network() 
  {
    try {
      if (network_role.equals("CLIENT")) {  // just a client
        port = Integer.parseInt(chosen_port);
        // reach out and touch someone
        the_game.mp = new 
          MessagePasser(the_game, chosen_hostname, port);
        the_game.mp.start();
      }
    }
    catch (IOException e) {
      System.err.println("Network Error! " + e);
      the_game.dispose();
    }
  }

  /** Starts the thread of this class going: */

  public void start()
  {
    this.setVisible(true);
  }

}



//////////////////////////////////////////////////////////
// Don't think, just hit `y':
//
// local variables:
// eval: (load-file "bkgomagic.el")
// end:
//////////////////////////////////////////////////////////
